'''///////////////////////////////////////////////////////////////////////////////
'	DataTypes.cs
'	Managed Wiimote Library
'	Written by Brian Peek (http://www.brianpeek.com/)
'	for MSDN's Coding4Fun (http://msdn.microsoft.com/coding4fun/)
'	Visit http://blogs.msdn.com/coding4fun/archive/2007/03/14/1879033.aspx
'	for more information
'''///////////////////////////////////////////////////////////////////////////////


' if we're building the MSRS version, we need to bring in the MSRS Attributes
' if we're not doing the MSRS build then define some fake attribute classes for DataMember/DataContract
#If MSRS Then
Imports Microsoft.Dss.Core.Attributes
#Else
NotInheritable Class DataContract
	Inherits Attribute
End Class

NotInheritable Class DataMember
	Inherits Attribute
End Class
#End If

Namespace WiimoteLib
	#If MSRS Then
	<DataContract> _
	Public Structure RumbleRequest
		<DataMember> _
		Public Rumble As Boolean
	End Structure
	#End If

	''' <summary>
	''' Current overall state of the Wiimote and all attachments
	''' </summary>
	<DataContract> _
	Public Class WiimoteState
		''' <summary>
		''' Current calibration information
		''' </summary>
		<DataMember> _
		Public AccelCalibrationInfo As New AccelCalibrationInfo()
		''' <summary>
		''' Current state of buttons
		''' </summary>
		<DataMember> _
		Public ButtonState As New ButtonState()
		''' <summary>
		''' Current state of accelerometers
		''' </summary>
		<DataMember> _
		Public AccelState As New AccelState()
		''' <summary>
		''' Current state of IR sensors
		''' </summary>
		<DataMember> _
		Public IRState As New IRState()
		''' <summary>
		''' Current battery level
		''' </summary>
		<DataMember> _
		Public Battery As Byte
		''' <summary>
		''' Current state of rumble
		''' </summary>
		<DataMember> _
		Public Rumble As Boolean
		''' <summary>
		''' Is an extension controller inserted?
		''' </summary>
		<DataMember> _
		Public Extension As Boolean
		''' <summary>
		''' Extension controller currently inserted, if any
		''' </summary>
		<DataMember> _
		Public ExtensionType As ExtensionType
		''' <summary>
		''' Current state of Nunchuk extension
		''' </summary>
		<DataMember> _
		Public NunchukState As New NunchukState()
		''' <summary>
		''' Current state of Classic Controller extension
		''' </summary>
		<DataMember> _
		Public ClassicControllerState As New ClassicControllerState()
		''' <summary>
		''' Current state of LEDs
		''' </summary>
		<DataMember> _
		Public LEDState As LEDState
	End Class

	''' <summary>
	''' Current state of LEDs
	''' </summary>
	<DataContract> _
	Public Structure LEDState
		''' <summary>
		''' LED on the Wiimote
		''' </summary>
		<DataMember> _
		Public LED1 As Boolean, LED2 As Boolean, LED3 As Boolean, LED4 As Boolean
	End Structure

	''' <summary>
	''' Calibration information stored on the Nunchuk
	''' </summary>
	<DataContract> _
	Public Structure NunchukCalibrationInfo
		''' <summary>
		''' Accelerometer calibration data
		''' </summary>
		Public AccelCalibration As AccelCalibrationInfo
		''' <summary>
		''' Joystick X-axis calibration
		''' </summary>
		<DataMember> _
		Public MinX As Byte, MidX As Byte, MaxX As Byte
		''' <summary>
		''' Joystick Y-axis calibration
		''' </summary>
		<DataMember> _
		Public MinY As Byte, MidY As Byte, MaxY As Byte
	End Structure

	''' <summary>
	''' Calibration information stored on the Classic Controller
	''' </summary>
	<DataContract> _
	Public Structure ClassicControllerCalibrationInfo
		''' <summary>
		''' Left joystick X-axis 
		''' </summary>
		<DataMember> _
		Public MinXL As Byte, MidXL As Byte, MaxXL As Byte
		''' <summary>
		''' Left joystick Y-axis
		''' </summary>
		<DataMember> _
		Public MinYL As Byte, MidYL As Byte, MaxYL As Byte
		''' <summary>
		''' Right joystick X-axis
		''' </summary>
		<DataMember> _
		Public MinXR As Byte, MidXR As Byte, MaxXR As Byte
		''' <summary>
		''' Right joystick Y-axis
		''' </summary>
		<DataMember> _
		Public MinYR As Byte, MidYR As Byte, MaxYR As Byte
		''' <summary>
		''' Left analog trigger
		''' </summary>
		<DataMember> _
		Public MinTriggerL As Byte, MaxTriggerL As Byte
		''' <summary>
		''' Right analog trigger
		''' </summary>
		<DataMember> _
		Public MinTriggerR As Byte, MaxTriggerR As Byte
	End Structure

	''' <summary>
	''' Current state of the Nunchuk extension
	''' </summary>
	<DataContract> _
	Public Structure NunchukState
		''' <summary>
		''' Calibration data for Nunchuk extension
		''' </summary>
		<DataMember> _
		Public CalibrationInfo As NunchukCalibrationInfo
		''' <summary>
		''' State of accelerometers
		''' </summary>
		<DataMember> _
		Public AccelState As AccelState
		''' <summary>
		''' Raw joystick position before normalization.  Values range between 0 and 255.
		''' </summary>
		<DataMember> _
		Public RawX As Byte, RawY As Byte
		''' <summary>
		''' Normalized joystick position.  Values range between -0.5 and 0.5
		''' </summary>
		<DataMember> _
		Public X As Single, Y As Single
		''' <summary>
		''' Digital button on Nunchuk extension
		''' </summary>
		<DataMember> _
		Public C As Boolean, Z As Boolean
	End Structure

	''' <summary>
	''' Curernt button state of the Classic Controller
	''' </summary>
	<DataContract> _
	Public Structure ClassicControllerButtonState
		''' <summary>
		''' Digital button on the Classic Controller extension
		''' </summary>
		<DataMember> _
		Public A As Boolean, B As Boolean, Plus As Boolean, Home As Boolean, Minus As Boolean, Up As Boolean, _
			Down As Boolean, Left As Boolean, Right As Boolean, X As Boolean, Y As Boolean, ZL As Boolean, _
			ZR As Boolean
		''' <summary>
		''' Analog trigger - false if released, true for any pressure applied
		''' </summary>
		<DataMember> _
		Public TriggerL As Boolean, TriggerR As Boolean
	End Structure

	''' <summary>
	''' Current state of the Classic Controller
	''' </summary>
	<DataContract> _
	Public Structure ClassicControllerState
		''' <summary>
		''' Calibration data for Classic Controller extension
		''' </summary>
		<DataMember> _
		Public CalibrationInfo As ClassicControllerCalibrationInfo
		''' <summary>
		''' Current button state
		''' </summary>
		<DataMember> _
		Public ButtonState As ClassicControllerButtonState
		''' <summary>
		''' Raw value of left joystick.  Values range between 0 - 255.
		''' </summary>
		<DataMember> _
		Public RawXL As Byte, RawYL As Byte
		''' <summary>
		''' Raw value of right joystick.  Values range between 0 - 255.
		''' </summary>
		<DataMember> _
		Public RawXR As Byte, RawYR As Byte
		''' <summary>
		''' Normalized value of left joystick.  Values range between -0.5 - 0.5
		''' </summary>
		<DataMember> _
		Public XL As Single, YL As Single
		''' <summary>
		''' Normalized value of right joystick.  Values range between -0.5 - 0.5
		''' </summary>
		<DataMember> _
		Public XR As Single, YR As Single
		''' <summary>
		''' Raw value of analog trigger.  Values range between 0 - 255.
		''' </summary>
		<DataMember> _
		Public RawTriggerL As Byte, RawTriggerR As Byte
		''' <summary>
		''' Normalized value of analog trigger.  Values range between 0.0 - 1.0
		''' </summary>
		<DataMember> _
		Public TriggerL As Single, TriggerR As Single
	End Structure

	''' <summary>
	''' Current state of the IR camera
	''' </summary>
	<DataContract> _
	Public Structure IRState
		''' <summary>
		''' Current mode of IR sensor data
		''' </summary>
		<DataMember> _
		Public Mode As IRMode
		''' <summary>
		''' Raw value of X-axis on individual sensor.  Values range between 0 - 1023
		''' </summary>
		<DataMember> _
		Public RawX1 As Integer, RawX2 As Integer, RawX3 As Integer, RawX4 As Integer
		''' <summary>
		''' Raw value of Y-axis on individual sensor.  Values range between 0 - 767
		''' </summary>
		<DataMember> _
		Public RawY1 As Integer, RawY2 As Integer, RawY3 As Integer, RawY4 As Integer
		''' <summary>
		''' Size of IR Sensor.  Values range from 0 - 15
		''' </summary>
		<DataMember> _
		Public Size1 As Integer, Size2 As Integer, Size3 As Integer, Size4 As Integer
		''' <summary>
		''' IR sensor seen
		''' </summary>
		<DataMember> _
		Public Found1 As Boolean, Found2 As Boolean, Found3 As Boolean, Found4 As Boolean
		''' <summary>
		''' Normalized value of X-axis on individual sensor.  Values range between 0.0 - 1.0
		''' </summary>
		<DataMember> _
		Public X1 As Single, X2 As Single, X3 As Single, X4 As Single
		''' <summary>
		''' Normalized value of Y-axis on individual sensor.  Values range between 0.0 - 1.0
		''' </summary>
		<DataMember> _
		Public Y1 As Single, Y2 As Single, Y3 As Single, Y4 As Single
		''' <summary>
		''' Raw midpoint of IR sensors 1 and 2 only.  Values range between 0 - 1023, 0 - 767
		''' </summary>
		<DataMember> _
		Public RawMidX As Integer, RawMidY As Integer
		''' <summary>
		''' Normalized midpoint of IR sensors 1 and 2 only.  Values range between 0.0 - 1.0
		''' </summary>
		<DataMember> _
		Public MidX As Single, MidY As Single
	End Structure

	''' <summary>
	''' Current state of the accelerometers
	''' </summary>
	<DataContract> _
	Public Structure AccelState
		''' <summary>
		''' Raw accelerometer data.
		''' <remarks>Values range between 0 - 255</remarks>
		''' </summary>
		<DataMember> _
		Public RawX As Byte, RawY As Byte, RawZ As Byte
		''' <summary>
		''' Normalized acceerometer data.  Values range between 0 - ?
		''' </summary>
		<DataMember> _
		Public X As Single, Y As Single, Z As Single
	End Structure

	''' <summary>
	''' Accelerometer calibration information
	''' </summary>
	<DataContract> _
	Public Structure AccelCalibrationInfo
		''' <summary>
		''' Zero point of accelerometer
		''' </summary>
		<DataMember> _
		Public X0 As Byte, Y0 As Byte, Z0 As Byte
		''' <summary>
		''' Gravity at rest of accelerometer
		''' </summary>
		<DataMember> _
		Public XG As Byte, YG As Byte, ZG As Byte
	End Structure

	''' <summary>
	''' Current button state
	''' </summary>
	<DataContract> _
	Public Structure ButtonState
		''' <summary>
		''' Digital button on the Wiimote
		''' </summary>
		<DataMember> _
		Public A As Boolean, B As Boolean, Plus As Boolean, Home As Boolean, Minus As Boolean, One As Boolean, _
			Two As Boolean, Up As Boolean, Down As Boolean, Left As Boolean, Right As Boolean
	End Structure

	''' <summary>
	''' The extension plugged into the Wiimote
	''' </summary>
	<DataContract> _
	Public Enum ExtensionType As Byte
		''' <summary>
		''' No extension
		''' </summary>
		None = &H0
		''' <summary>
		''' Nunchuk extension
		''' </summary>
		Nunchuk = &Hfe
		''' <summary>
		''' Classic Controller extension
		''' </summary>
		ClassicController = &Hfd
	End Enum

	''' <summary>
	''' The mode of data reported for the IR sensor
	''' </summary>
	<DataContract> _
	Public Enum IRMode As Byte
		''' <summary>
		''' IR sensor off
		''' </summary>
		Off = &H0
		''' <summary>
		''' Basic mode
		''' </summary>
		Basic = &H1
		' 10 bytes
		''' <summary>
		''' Extended mode
		''' </summary>
		Extended = &H3
		' 12 bytes
		''' <summary>
		''' Full mode (unsupported)
		''' </summary>
		Full = &H5
		' 16 bytes * 2 (format unknown)
	End Enum
End Namespace
