﻿Imports System.Collections.Generic
Imports System.ComponentModel
Imports System.Drawing
Imports System.Drawing.Imaging
Imports System.Text
Imports System.Windows.Forms
Imports System.Diagnostics
Imports System.Threading
Imports System.Runtime.InteropServices
'for firing keyboard and mouse events (optional)
Imports System.IO
'for saving the reading the calibration data

Imports WiimoteActiveboard_2_VB.WiimoteLib

Public Class Form1
    Inherits Form
    'instance of the wii remote
    Private wm As New Wiimote()

    Const smoothingBufferSize As Integer = 50

    Private smoothingBuffer As PointF() = New PointF(smoothingBufferSize - 1) {}
    Private smoothingBufferIndex As Integer = 0
    Private smoothingAmount As Integer = 4
    Private enableSmoothing As Boolean = True

    Private cursorControl As Boolean = False

    Private screenWidth As Integer = 1024
    'defaults, gets replaced by actual screen size
    Private screenHeight As Integer = 768

    Private calibrationState As Integer = 0
    Private calibrationMargin As Single = 0.1F

    Private cf As CalibrationForm = Nothing

    Private warper As New WiimoteWhiteboard.Warper()
    Private srcX As Single() = New Single(3) {}
    Private srcY As Single() = New Single(3) {}
    Private dstX As Single() = New Single(3) {}
    Private dstY As Single() = New Single(3) {}


    'declare consts for mouse messages
    Public Const INPUT_MOUSE As Integer = 0
    Public Const INPUT_KEYBOARD As Integer = 1
    Public Const INPUT_HARDWARE As Integer = 2

    Public Const MOUSEEVENTF_MOVE As Integer = &H1
    Public Const MOUSEEVENTF_LEFTDOWN As Integer = &H2
    Public Const MOUSEEVENTF_LEFTUP As Integer = &H4
    Public Const MOUSEEVENTF_RIGHTDOWN As Integer = &H8
    Public Const MOUSEEVENTF_RIGHTUP As Integer = &H10
    Public Const MOUSEEVENTF_MIDDLEDOWN As Integer = &H20
    Public Const MOUSEEVENTF_MIDDLEUP As Integer = &H40
    Public Const MOUSEEVENTF_ABSOLUTE As Integer = &H8000

    'declare consts for key scan codes
    Public Const VK_TAB As Byte = &H9
    Public Const VK_MENU As Byte = &H12
    ' VK_MENU is Microsoft talk for the ALT key
    Public Const VK_SPACE As Byte = &H20
    Public Const VK_RETURN As Byte = &HD
    Public Const VK_LEFT As Byte = &H25
    Public Const VK_UP As Byte = &H26
    Public Const VK_RIGHT As Byte = &H27
    Public Const VK_DOWN As Byte = &H28
    Public Const KEYEVENTF_EXTENDEDKEY As Integer = &H1
    Public Const KEYEVENTF_KEYUP As Integer = &H2

    'for firing mouse and keyboard events
    <DllImport("user32.dll", SetLastError:=True)> _
    Private Shared Function SendInput(ByVal nInputs As UInteger, ByVal pInputs As INPUT(), ByVal cbSize As Integer) As UInteger
    End Function

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure MOUSEINPUT
        Public dx As Integer
        '4
        Public dy As Integer
        '4
        Public mouseData As UInteger
        '4
        Public dwFlags As UInteger
        '4
        Public time As UInteger
        '4
        Public dwExtraInfo As IntPtr
        '4
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure KEYBDINPUT
        Public wVk As UShort
        '2
        Public wScan As UShort
        '2
        Public dwFlags As UInteger
        '4
        Public time As UInteger
        '4
        Public dwExtraInfo As IntPtr
        '4
    End Structure

    <StructLayout(LayoutKind.Sequential)> _
    Public Structure HARDWAREINPUT
        Public uMsg As UInteger
        Public wParamL As UShort
        Public wParamH As UShort
    End Structure

    <StructLayout(LayoutKind.Explicit, Size:=28)> _
    Public Structure INPUT
        <FieldOffset(0)> _
        Public type As Integer
        '*
        <FieldOffset(4)> _
        Public mi As MOUSEINPUT
        '*
        <FieldOffset(4)> _
        Public ki As KEYBDINPUT
        '*
        <FieldOffset(4)> _
        Public hi As HARDWAREINPUT
    End Structure
    'imports mouse_event function from user32.dll

    ' motion and click options
    ' horizontal position or change
    ' vertical position or change
    ' wheel movement
    <DllImport("user32.dll")> _
    Private Shared Sub mouse_event(ByVal dwFlags As Long, ByVal dx As Long, ByVal dy As Long, ByVal dwData As Long, ByVal dwExtraInfo As Long)
        ' application-defined information
    End Sub

    <DllImport("user32.dll", SetLastError:=True)> _
    Public Shared Function SetCursorPos(ByVal X As Integer, ByVal Y As Integer) As <MarshalAs(UnmanagedType.Bool)> Boolean
    End Function

    'imports keybd_event function from user32.dll
    <DllImport("user32.dll", CharSet:=CharSet.Auto, CallingConvention:=CallingConvention.StdCall)> _
    Public Shared Sub keybd_event(ByVal bVk As Byte, ByVal bScan As Byte, ByVal dwFlags As Long, ByVal dwExtraInfo As Long)
    End Sub
    Private lastWiiState As New WiimoteState()
    'helps with event firing
    'end keyboard and mouse input emulation variables----------------------------------------

    Private mut As New Mutex()

    Public Sub New()
        screenWidth = Screen.GetBounds(Me).Width
        screenHeight = Screen.GetBounds(Me).Height
        InitializeComponent()

        For i As Integer = 0 To smoothingBufferSize - 1
            smoothingBuffer(i) = New PointF()
        Next

        setSmoothing(smoothingAmount)
    End Sub

    Private Sub wm_OnWiimoteExtensionChanged(ByVal sender As Object, ByVal args As WiimoteExtensionChangedEventArgs)

        'if extension attached, enable it
        If args.Inserted Then
            wm.SetReportType(Wiimote.InputReport.IRExtensionAccel, True)
        Else
            wm.SetReportType(Wiimote.InputReport.IRAccel, True)
        End If
    End Sub

    Private Function UpdateTrackingUtilization() As Single
        'area of ideal calibration coordinates (to match the screen)
        Dim idealArea As Single = (1 - 2 * calibrationMargin) * 1024 * (1 - 2 * calibrationMargin) * 768

        'area of quadrliatera
        Dim actualArea As Single = 0.5F * Math.Abs((srcX(1) - srcX(2)) * (srcY(0) - srcY(3)) - (srcX(0) - srcX(3)) * (srcY(1) - srcY(2)))
        Dim util As Single = (actualArea / idealArea) * 100
        BeginInvoke(DirectCast(Sub() Settings.lblTrackingUtil.Text = util.ToString("f0"), MethodInvoker))
        BeginInvoke(DirectCast(Sub() Settings.pbTrackingUtil.Value = CInt(Math.Truncate(util)), MethodInvoker))

        Return util

    End Function

    Private Function getSmoothedCursor(ByVal amount As Integer) As PointF
        Dim start As Integer = smoothingBufferIndex - amount
        If start < 0 Then
            start = 0
        End If
        Dim smoothed As New PointF(0, 0)
        Dim count As Integer = smoothingBufferIndex - start
        For i As Integer = start To smoothingBufferIndex - 1
            smoothed.X += smoothingBuffer(i Mod smoothingBufferSize).X
            smoothed.Y += smoothingBuffer(i Mod smoothingBufferSize).Y
        Next
        smoothed.X /= count
        smoothed.Y /= count
        Return smoothed
    End Function

    Private Sub wm_OnWiimoteChanged(ByVal sender As Object, ByVal args As WiimoteChangedEventArgs)
        mut.WaitOne()

        'extract the wiimote state
        Dim ws As WiimoteState = args.WiimoteState

        If ws.IRState.Found1 Then
            Dim x As Integer = ws.IRState.RawX1
            Dim y As Integer = ws.IRState.RawY1
            Dim warpedX As Single = x
            Dim warpedY As Single = y
            warper.warp(x, y, warpedX, warpedY)

            smoothingBuffer(smoothingBufferIndex Mod smoothingBufferSize).X = warpedX
            smoothingBuffer(smoothingBufferIndex Mod smoothingBufferSize).Y = warpedY
            smoothingBufferIndex += 1

            If Not lastWiiState.IRState.Found1 Then
                'mouse down
                lastWiiState.IRState.Found1 = ws.IRState.Found1
                smoothingBufferIndex = 0
                'resets the count
                If cursorControl Then
                    Dim buffer As INPUT() = New INPUT(1) {}
                    buffer(0).type = INPUT_MOUSE
                    buffer(0).mi.dx = CInt(Math.Truncate(warpedX * 65535.0F / screenWidth))
                    buffer(0).mi.dy = CInt(Math.Truncate(warpedY * 65535.0F / screenHeight))
                    buffer(0).mi.mouseData = 0
                    buffer(0).mi.dwFlags = MOUSEEVENTF_ABSOLUTE Or MOUSEEVENTF_MOVE
                    buffer(0).mi.time = 0
                    buffer(0).mi.dwExtraInfo = CType(0, IntPtr)

                    buffer(1).type = INPUT_MOUSE
                    buffer(1).mi.dx = 0
                    buffer(1).mi.dy = 0
                    buffer(1).mi.mouseData = 0
                    buffer(1).mi.dwFlags = MOUSEEVENTF_LEFTDOWN
                    buffer(1).mi.time = 1
                    buffer(1).mi.dwExtraInfo = CType(0, IntPtr)


                    SendInput(2, buffer, Marshal.SizeOf(buffer(0)))
                End If
                'cusor control
                Select Case calibrationState
                    Case 1
                        srcX(calibrationState - 1) = x
                        srcY(calibrationState - 1) = y
                        calibrationState = 2
                        doCalibration()
                        Exit Select
                    Case 2
                        srcX(calibrationState - 1) = x
                        srcY(calibrationState - 1) = y
                        calibrationState = 3
                        doCalibration()
                        Exit Select
                    Case 3
                        srcX(calibrationState - 1) = x
                        srcY(calibrationState - 1) = y
                        calibrationState = 4
                        doCalibration()
                        Exit Select
                    Case 4
                        srcX(calibrationState - 1) = x
                        srcY(calibrationState - 1) = y
                        calibrationState = 5
                        doCalibration()
                        Exit Select
                    Case Else
                        Exit Select
                        'calibtation state
                End Select
            Else
                'mouse down                
                If cursorControl Then
                    'dragging
                    Dim buffer As INPUT() = New INPUT(0) {}
                    buffer(0).type = INPUT_MOUSE
                    If enableSmoothing Then
                        Dim s As PointF = getSmoothedCursor(smoothingAmount)
                        buffer(0).mi.dx = CInt(Math.Truncate(s.X * 65535.0F / screenWidth))
                        buffer(0).mi.dy = CInt(Math.Truncate(s.Y * 65535.0F / screenHeight))
                    Else
                        buffer(0).mi.dx = CInt(Math.Truncate(warpedX * 65535.0F / screenWidth))
                        buffer(0).mi.dy = CInt(Math.Truncate(warpedY * 65535.0F / screenHeight))
                    End If
                    buffer(0).mi.mouseData = 0
                    buffer(0).mi.dwFlags = MOUSEEVENTF_ABSOLUTE Or MOUSEEVENTF_MOVE
                    buffer(0).mi.time = 0
                    buffer(0).mi.dwExtraInfo = CType(0, IntPtr)

                    SendInput(1, buffer, Marshal.SizeOf(buffer(0)))
                End If
            End If
        Else
            'ir visible
            If lastWiiState.IRState.Found1 Then
                'mouse up
                lastWiiState.IRState.Found1 = ws.IRState.Found1
                If cursorControl Then
                    Dim buffer As INPUT() = New INPUT(1) {}
                    buffer(0).type = INPUT_MOUSE
                    buffer(0).mi.dx = 0
                    buffer(0).mi.dy = 0
                    buffer(0).mi.mouseData = 0
                    buffer(0).mi.dwFlags = MOUSEEVENTF_LEFTUP
                    buffer(0).mi.time = 0
                    buffer(0).mi.dwExtraInfo = CType(0, IntPtr)

                    buffer(1).type = INPUT_MOUSE
                    buffer(1).mi.dx = 0
                    buffer(1).mi.dy = 0
                    buffer(1).mi.mouseData = 0
                    buffer(1).mi.dwFlags = MOUSEEVENTF_MOVE
                    buffer(1).mi.time = 0
                    buffer(1).mi.dwExtraInfo = CType(0, IntPtr)


                    SendInput(2, buffer, Marshal.SizeOf(buffer(0)))
                End If
                'ir lost
            End If
        End If

        If Not lastWiiState.ButtonState.A AndAlso ws.ButtonState.A Then
            If My.Settings.key_a = "Calibrate" Then
                BeginInvoke(DirectCast(Sub() TSB_Calibrate.PerformClick(), MethodInvoker))
            ElseIf My.Settings.key_a = "ENTER" Then
                keybd_event(System.Windows.Forms.Keys.Enter, &H45, 0, 0)
            ElseIf My.Settings.key_a = "RETURN" Then
                keybd_event(System.Windows.Forms.Keys.Return, &H45, 0, 0)
            Else
                Try
                    Process.Start(My.Settings.key_a)
                Catch ex As Exception

                End Try
            End If
        End If
        If lastWiiState.ButtonState.A AndAlso Not ws.ButtonState.A Then
            If My.Settings.key_a = "ENTER" Then
                keybd_event(System.Windows.Forms.Keys.Enter, &H45, KEYEVENTF_KEYUP, 0)
            ElseIf My.Settings.key_a = "RETURN" Then
                keybd_event(System.Windows.Forms.Keys.Return, &H45, KEYEVENTF_KEYUP, 0)
            End If
        End If
        lastWiiState.ButtonState.A = ws.ButtonState.A

        If Not lastWiiState.ButtonState.B AndAlso ws.ButtonState.B Then
            If My.Settings.key_b = "Calibrate" Then
                BeginInvoke(DirectCast(Sub() TSB_Calibrate.PerformClick(), MethodInvoker))
            ElseIf My.Settings.key_b = "ENTER" Then
                keybd_event(System.Windows.Forms.Keys.Enter, &H45, 0, 0)
            ElseIf My.Settings.key_b = "RETURN" Then
                keybd_event(System.Windows.Forms.Keys.Return, &H45, 0, 0)
            Else
                Try
                    Process.Start(My.Settings.key_b)
                Catch ex As Exception

                End Try
            End If
        End If
        If lastWiiState.ButtonState.B AndAlso Not ws.ButtonState.B Then
            If My.Settings.key_b = "ENTER" Then
                keybd_event(System.Windows.Forms.Keys.Enter, &H45, KEYEVENTF_KEYUP, 0)
            ElseIf My.Settings.key_b = "RETURN" Then
                keybd_event(System.Windows.Forms.Keys.Return, &H45, KEYEVENTF_KEYUP, 0)
            End If
        End If
        lastWiiState.ButtonState.B = ws.ButtonState.B

        If Not lastWiiState.ButtonState.Up AndAlso ws.ButtonState.Up Then
            keybd_event(VK_UP, &H45, 0, 0)
        End If
        If lastWiiState.ButtonState.Up AndAlso Not ws.ButtonState.Up Then
            keybd_event(VK_UP, &H45, KEYEVENTF_KEYUP, 0)
        End If
        lastWiiState.ButtonState.Up = ws.ButtonState.Up

        If Not lastWiiState.ButtonState.Down AndAlso ws.ButtonState.Down Then
            keybd_event(VK_DOWN, &H45, 0, 0)
        End If
        If lastWiiState.ButtonState.Down AndAlso Not ws.ButtonState.Down Then
            keybd_event(VK_DOWN, &H45, KEYEVENTF_KEYUP, 0)
        End If
        lastWiiState.ButtonState.Down = ws.ButtonState.Down

        If Not lastWiiState.ButtonState.Left AndAlso ws.ButtonState.Left Then
            keybd_event(VK_LEFT, &H45, 0, 0)
        End If
        If lastWiiState.ButtonState.Left AndAlso Not ws.ButtonState.Left Then
            keybd_event(VK_LEFT, &H45, KEYEVENTF_KEYUP, 0)
        End If
        lastWiiState.ButtonState.Left = ws.ButtonState.Left

        If Not lastWiiState.ButtonState.Right AndAlso ws.ButtonState.Right Then
            keybd_event(VK_RIGHT, &H45, 0, 0)
        End If
        If lastWiiState.ButtonState.Right AndAlso Not ws.ButtonState.Right Then
            keybd_event(VK_RIGHT, &H45, KEYEVENTF_KEYUP, 0)
        End If
        lastWiiState.ButtonState.Right = ws.ButtonState.Right

        'NEW
        If Not lastWiiState.ButtonState.Plus AndAlso ws.ButtonState.Plus Then
            If My.Settings.key_plus = "Volume up" Then
                keybd_event(System.Windows.Forms.Keys.VolumeUp, &H45, 0, 0)
            ElseIf My.Settings.key_plus = "Zoom in" Then

            Else
                Try
                    Process.Start(My.Settings.key_plus)
                Catch ex As Exception

                End Try
            End If
        End If
        If lastWiiState.ButtonState.Plus AndAlso Not ws.ButtonState.Plus Then
            If My.Settings.key_plus = "Volume up" Then
                keybd_event(System.Windows.Forms.Keys.VolumeUp, &H45, KEYEVENTF_KEYUP, 0)
            ElseIf My.Settings.key_plus = "Zoom in" Then

            End If
        End If
            lastWiiState.ButtonState.Plus = ws.ButtonState.Plus

        If Not lastWiiState.ButtonState.Minus AndAlso ws.ButtonState.Minus Then
            If My.Settings.key_minus = "Volume up" Then
                keybd_event(System.Windows.Forms.Keys.VolumeDown, &H45, 0, 0)
            ElseIf My.Settings.key_minus = "Zoom in" Then

            Else
                Try
                    Process.Start(My.Settings.key_minus)
                Catch ex As Exception

                End Try
            End If
        End If
        If lastWiiState.ButtonState.Minus AndAlso Not ws.ButtonState.Minus Then
            If My.Settings.key_minus = "Volume up" Then
                keybd_event(System.Windows.Forms.Keys.VolumeDown, &H45, KEYEVENTF_KEYUP, 0)
            ElseIf My.Settings.key_minus = "Zoom in" Then

            End If
        End If
        lastWiiState.ButtonState.Minus = ws.ButtonState.Minus

        If Not lastWiiState.ButtonState.Home AndAlso ws.ButtonState.Home Then
            If My.Settings.key_home = "Volume mute" Then
                keybd_event(System.Windows.Forms.Keys.VolumeMute, &H45, 0, 0)
            Else
                Try
                    Process.Start(My.Settings.key_home)
                Catch ex As Exception

                End Try
            End If
        End If
        If lastWiiState.ButtonState.Home AndAlso Not ws.ButtonState.Home Then
            If My.Settings.key_home = "Volume mute" Then
                keybd_event(System.Windows.Forms.Keys.VolumeMute, &H45, KEYEVENTF_KEYUP, 0)
            ElseIf My.Settings.key_home = "Zoom in" Then

            End If
        End If
        lastWiiState.ButtonState.Home = ws.ButtonState.Home

        If Not lastWiiState.ButtonState.One AndAlso ws.ButtonState.One Then
            Try
                Process.Start(My.Settings.key_1)
            Catch ex As Exception

            End Try
        End If
        lastWiiState.ButtonState.One = ws.ButtonState.One

        If Not lastWiiState.ButtonState.Two AndAlso ws.ButtonState.Two Then
            Try
                Process.Start(My.Settings.key_2)
            Catch ex As Exception

            End Try
        End If
        lastWiiState.ButtonState.Two = ws.ButtonState.Two
        'END NEW

            lastWiiState.IRState.Found1 = ws.IRState.Found1
            lastWiiState.IRState.RawX1 = ws.IRState.RawX1
            lastWiiState.IRState.RawY1 = ws.IRState.RawY1
            lastWiiState.IRState.Found2 = ws.IRState.Found2
            lastWiiState.IRState.RawX2 = ws.IRState.RawX2
            lastWiiState.IRState.RawY2 = ws.IRState.RawY2
            lastWiiState.IRState.Found3 = ws.IRState.Found3
            lastWiiState.IRState.RawX3 = ws.IRState.RawX3
            lastWiiState.IRState.RawY3 = ws.IRState.RawY3
            lastWiiState.IRState.Found4 = ws.IRState.Found4
            lastWiiState.IRState.RawX4 = ws.IRState.RawX4
            lastWiiState.IRState.RawY4 = ws.IRState.RawY4

            'draw battery value on GUI
            BeginInvoke(DirectCast(Sub() PB_Battery.Value = (If(ws.Battery > &HC8, &HC8, CInt(ws.Battery))), MethodInvoker))
            Dim f As Single = (((100.0F * 48.0F * CSng(ws.Battery / 48.0F))) / 192.0F)
            BeginInvoke(DirectCast(Sub() TSL_Battery_percent.Text = f.ToString("f0") & "%", MethodInvoker))

            'check the GUI check boxes if the IR dots are visible
            Dim irstatus As [String] = lang.current_ir_string
            If ws.IRState.Found1 Then
                irstatus += "1 "
            End If
            If ws.IRState.Found2 Then
                irstatus += "2 "
            End If
            If ws.IRState.Found3 Then
                irstatus += "3 "
            End If
            If ws.IRState.Found4 Then
                irstatus += "4 "
            End If

            BeginInvoke(DirectCast(Sub() lblIRvisible.Text = irstatus, MethodInvoker))

            mut.ReleaseMutex()
    End Sub


    Public Sub loadCalibrationData()
        ' create reader & open file
        Try
            Dim tr As TextReader = New StreamReader("calibration.dat")
            For i As Integer = 0 To 3
                srcX(i) = Single.Parse(tr.ReadLine())
                srcY(i) = Single.Parse(tr.ReadLine())
            Next
            smoothingAmount = Integer.Parse(tr.ReadLine())

            ' close the stream
            tr.Close()
        Catch x As Exception
            'no prexsting calibration
            Return
        End Try

        warper.setDestination(screenWidth * calibrationMargin, screenHeight * calibrationMargin, screenWidth * (1.0F - calibrationMargin), screenHeight * calibrationMargin, screenWidth * calibrationMargin, screenHeight * (1.0F - calibrationMargin), _
         screenWidth * (1.0F - calibrationMargin), screenHeight * (1.0F - calibrationMargin))
        warper.setSource(srcX(0), srcY(0), srcX(1), srcY(1), srcX(2), srcY(2), _
         srcX(3), srcY(3))

        warper.computeWarp()

        setSmoothing(smoothingAmount)

        cursorControl = True
        BeginInvoke(DirectCast(Sub() TSB_CursorControl.Checked = cursorControl, MethodInvoker))

        UpdateTrackingUtilization()
    End Sub

    Public Sub saveCalibrationData()
        Dim tw As TextWriter = New StreamWriter("calibration.dat")

        ' write a line of text to the file
        For i As Integer = 0 To 3
            tw.WriteLine(srcX(i))
            tw.WriteLine(srcY(i))
        Next
        tw.WriteLine(smoothingAmount)
        ' close the stream
        tw.Close()
    End Sub

    Public Sub doCalibration()
        If cf Is Nothing Then
            Return
        End If
        Dim x As Integer = 0
        Dim y As Integer = 0
        Dim size As Integer = 25
        Dim p As New Pen(Color.Red)
        Select Case calibrationState
            Case 1
                x = CInt(Math.Truncate(screenWidth * calibrationMargin))
                y = CInt(Math.Truncate(screenHeight * calibrationMargin))
                cf.showCalibration(x, y, size, p)
                dstX(calibrationState - 1) = x
                dstY(calibrationState - 1) = y
                Exit Select
            Case 2
                x = screenWidth - CInt(Math.Truncate(screenWidth * calibrationMargin))
                y = CInt(Math.Truncate(screenHeight * calibrationMargin))
                cf.showCalibration(x, y, size, p)
                dstX(calibrationState - 1) = x
                dstY(calibrationState - 1) = y
                Exit Select
            Case 3
                x = CInt(Math.Truncate(screenWidth * calibrationMargin))
                y = screenHeight - CInt(Math.Truncate(screenHeight * calibrationMargin))
                cf.showCalibration(x, y, size, p)
                dstX(calibrationState - 1) = x
                dstY(calibrationState - 1) = y
                Exit Select
            Case 4
                x = screenWidth - CInt(Math.Truncate(screenWidth * calibrationMargin))
                y = screenHeight - CInt(Math.Truncate(screenHeight * calibrationMargin))
                cf.showCalibration(x, y, size, p)
                dstX(calibrationState - 1) = x
                dstY(calibrationState - 1) = y
                Exit Select
            Case 5
                'compute warp
                warper.setDestination(dstX(0), dstY(0), dstX(1), dstY(1), dstX(2), dstY(2), _
                 dstX(3), dstY(3))
                warper.setSource(srcX(0), srcY(0), srcX(1), srcY(1), srcX(2), srcY(2), _
                 srcX(3), srcY(3))
                warper.computeWarp()
                cf.Close()
                cf = Nothing
                calibrationState = 0
                cursorControl = True
                BeginInvoke(DirectCast(Sub() TSB_CursorControl.Checked = cursorControl, MethodInvoker))
                '                    saveCalibrationData();
                UpdateTrackingUtilization()
                Me.WindowState = FormWindowState.Normal
                Exit Select
            Case Else
                Exit Select
        End Select

    End Sub

    Private Sub setSmoothing(ByVal smoothing As Integer)
        smoothingAmount = smoothing
        Settings.trackBar1.Value = smoothing
        enableSmoothing = (smoothingAmount <> 0)
        Settings.Grp_Smoothing.Text = "Cursor Smoothing: " & smoothingAmount
    End Sub

    Private Sub Form1_FormClosed(ByVal sender As Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles Me.FormClosed
        'disconnect the wiimote
        wm.Disconnect()
        saveCalibrationData()
    End Sub

    Public Sub accept_lang()
        If My.Settings.lang = 0 Then
            TSB_Calibrate.Text = lang.eng_calibrate
            TSB_Draw.Text = lang.eng_draw
            TSB_CursorControl.Text = lang.eng_cursor_control
            TSB_Settings.Text = lang.eng_settings
            TSB_About.Text = lang.eng_about
            TSL_Battery.Text = lang.eng_battery
            lang.current_ir_string = lang.eng_irdots
            Draw_Settings.TSL_PenColor.Text = lang.eng_PenColor
            Draw_Settings.TSL_PenWidth.Text = lang.eng_PenWidth
            Draw_Settings.TSL_BGColor.Text = lang.eng_BGColor
            Draw_Settings.TSB_Save.Text = lang.eng_save
            Draw_Settings.TSB_Open.Text = lang.eng_open
            Draw_Settings.TSB_Exit.Text = lang.eng_draw_exit
            Settings.Text = lang.eng_settings
            TSB_Draw_on_Desktop.Text = lang.eng_on_the_desktop
            TSB_Draw_on_Color.Text = lang.eng_color_as_background
            TSB_Calibrate.ToolTipText = lang.eng_tip_calibrate
            TSB_Draw.ToolTipText = lang.eng_tip_draw
            TSB_CursorControl.ToolTipText = lang.eng_tip_CursorControl
            TSB_About.ToolTipText = lang.eng_tip_about

        ElseIf My.Settings.lang = 1 Then
            TSB_Calibrate.Text = lang.deu_calibrate
            TSB_Draw.Text = lang.deu_draw
            TSB_CursorControl.Text = lang.deu_cursor_control
            TSB_Settings.Text = lang.deu_settings
            TSB_About.Text = lang.deu_about
            TSL_Battery.Text = lang.deu_battery
            lang.current_ir_string = lang.deu_irdots
            Draw_Settings.TSL_PenColor.Text = lang.deu_PenColor
            Draw_Settings.TSL_PenWidth.Text = lang.deu_PenWidth
            Draw_Settings.TSL_BGColor.Text = lang.deu_BGColor
            Draw_Settings.TSB_Save.Text = lang.deu_save
            Draw_Settings.TSB_Open.Text = lang.deu_open
            Draw_Settings.TSB_Exit.Text = lang.deu_draw_exit
            Settings.Text = lang.deu_settings
            TSB_Draw_on_Desktop.Text = lang.deu_on_the_desktop
            TSB_Draw_on_Color.Text = lang.deu_color_as_background
            TSB_Calibrate.ToolTipText = lang.deu_tip_calibrate
            TSB_Draw.ToolTipText = lang.deu_tip_draw
            TSB_CursorControl.ToolTipText = lang.deu_tip_CursorControl
            TSB_About.ToolTipText = lang.deu_tip_about

        End If
    End Sub

    Private Sub Form1_Load_2(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Me.Text = Me.Text & " " & Application.ProductVersion

        'add event listeners to changes in the wiiremote
        'fired for every input report - usually 100 times per second if acclerometer is enabled
        AddHandler wm.WiimoteChanged, AddressOf wm_OnWiimoteChanged
        'fired when the extension is attached on unplugged
        AddHandler wm.WiimoteExtensionChanged, AddressOf wm_OnWiimoteExtensionChanged

        Try
            'connect to wii remote
            wm.Connect()

            'set what features you want to enable for the remote, look at Wiimote.InputReport for options
            wm.SetReportType(Wiimote.InputReport.IRAccel, True)


            'set wiiremote LEDs with this enumerated ID
            wm.SetLEDs(True, False, False, False)
        Catch x As Exception
            MessageBox.Show("Exception: " & x.Message, "Wiimote initialization error", MessageBoxButtons.OK, MessageBoxIcon.Error)
            End
        End Try
        loadCalibrationData()
        accept_lang()

        Settings.Opacity = 0
        Settings.Show()
        Settings.Hide()
        Settings.Opacity = 1
    End Sub

    Public Sub execute_trackbarscroll(ByVal value As Integer)
        smoothingAmount = value
        enableSmoothing = (smoothingAmount <> 0)
    End Sub

    Private Sub TSB_Calibrate_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TSB_Calibrate.Click
        If cf Is Nothing Then
            cf = New CalibrationForm()
            cf.Show()
        End If
        If cf.IsDisposed Then
            cf = New CalibrationForm()
            cf.Show()
        End If
        cursorControl = False
        calibrationState = 1
        doCalibration()
        Me.WindowState = FormWindowState.Minimized
    End Sub

    Private Sub cbCursorControl_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TSB_CursorControl.Click
        cursorControl = TSB_CursorControl.Checked
    End Sub

    Private Sub TSB_Settings_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TSB_Settings.Click
        Settings.Show()
        Settings.BringToFront()
        accept_lang()
    End Sub

    Private Sub TSB_About_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles TSB_About.Click
        AboutBox.ShowDialog()
    End Sub

    Private Function GesamterDesk() As Image
        Dim bild As New Bitmap(SystemInformation.VirtualScreen.Width, SystemInformation.VirtualScreen.Height)
        Dim grafik As Graphics = Graphics.FromImage(bild)

        grafik.CopyFromScreen(0, 0, 0, 0, bild.Size)
        grafik.Dispose()

        Return bild
    End Function

    Private Sub TSB_Draw_ButtonClick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TSB_Draw.ButtonClick, TSB_Draw_on_Desktop.Click
        Me.Opacity = 0
        Dim img As Image = GesamterDesk()
        Me.Opacity = 1

        Drawer.Show()

        Drawer.HG_Bmp = img 'New Bitmap(img.Width, img.Height)
        Drawer.bmp = img 'New Bitmap(img.Width, img.Height)

        Drawer.PBDraw.Image = Drawer.HG_Bmp

        Draw_Settings.Show()
        accept_lang()

        Me.Hide()
    End Sub

    Private Sub WeißerHintergrundToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TSB_Draw_on_Color.Click
        Drawer.Show()

        Drawer.HG_Bmp = New Bitmap(Drawer.PBDraw.Width, Drawer.PBDraw.Height)
        Drawer.bmp = New Bitmap(Drawer.PBDraw.Width, Drawer.PBDraw.Height)
        Using gr As Graphics = Graphics.FromImage(Drawer.bmp)
            gr.Clear(Color.Transparent)
            Drawer.PBDraw.Invalidate()
        End Using
        Using gr As Graphics = Graphics.FromImage(Drawer.HG_Bmp)
            gr.Clear(Color.White)
            gr.DrawImage(Drawer.bmp, 0, 0)
            Drawer.PBDraw.Invalidate()
        End Using
        Drawer.PBDraw.Image = Drawer.HG_Bmp

        Draw_Settings.Show()
        accept_lang()

        Me.Hide()
    End Sub
End Class
