'''///////////////////////////////////////////////////////////////////////////////
'	HIDImports.cs
'	Managed Wiimote Library
'	Written by Brian Peek (http://www.brianpeek.com/)
'	for MSDN's Coding4Fun (http://msdn.microsoft.com/coding4fun/)
'	Visit http://blogs.msdn.com/coding4fun/archive/2007/03/14/1879033.aspx
'	for more information
'''///////////////////////////////////////////////////////////////////////////////

Imports System.Runtime.InteropServices
Imports System.IO
Imports Microsoft.Win32.SafeHandles

Namespace WiimoteLib
	''' <summary>
	''' Win32 import information for use with the Wiimote library
	''' </summary>
	Class HIDImports
		'
		' Flags controlling what is included in the device information set built
		' by SetupDiGetClassDevs
		'
		Public Const DIGCF_DEFAULT As Integer = &H1
		' only valid with DIGCF_DEVICEINTERFACE
		Public Const DIGCF_PRESENT As Integer = &H2
		Public Const DIGCF_ALLCLASSES As Integer = &H4
		Public Const DIGCF_PROFILE As Integer = &H8
		Public Const DIGCF_DEVICEINTERFACE As Integer = &H10

		<Flags> _
		Public Enum EFileAttributes As UInteger
			[Readonly] = &H1
			Hidden = &H2
			System = &H4
			Directory = &H10
			Archive = &H20
			Device = &H40
			Normal = &H80
			Temporary = &H100
			SparseFile = &H200
			ReparsePoint = &H400
			Compressed = &H800
			Offline = &H1000
			NotContentIndexed = &H2000
			Encrypted = &H4000
			Write_Through = &H80000000UI
			Overlapped = &H40000000
			NoBuffering = &H20000000
			RandomAccess = &H10000000
			SequentialScan = &H8000000
			DeleteOnClose = &H4000000
			BackupSemantics = &H2000000
			PosixSemantics = &H1000000
			OpenReparsePoint = &H200000
			OpenNoRecall = &H100000
			FirstPipeInstance = &H80000
		End Enum

		<StructLayout(LayoutKind.Sequential)> _
		Public Structure SP_DEVINFO_DATA
			Public cbSize As UInteger
			Public ClassGuid As Guid
			Public DevInst As UInteger
			Public Reserved As IntPtr
		End Structure

		<StructLayout(LayoutKind.Sequential)> _
		Public Structure SP_DEVICE_INTERFACE_DATA
			Public cbSize As Integer
			Public InterfaceClassGuid As Guid
			Public Flags As Integer
			Public RESERVED As IntPtr
		End Structure

		<StructLayout(LayoutKind.Sequential, Pack := 1)> _
		Public Structure SP_DEVICE_INTERFACE_DETAIL_DATA
			Public cbSize As UInt32
			<MarshalAs(UnmanagedType.ByValTStr, SizeConst := 256)> _
			Public DevicePath As String
		End Structure

		<StructLayout(LayoutKind.Sequential)> _
		Public Structure HIDD_ATTRIBUTES
			Public Size As Integer
			Public VendorID As Short
			Public ProductID As Short
			Public VersionNumber As Short
		End Structure

		<DllImport("hid.dll", CharSet := CharSet.Auto, SetLastError := True)> _
		Public Shared Sub HidD_GetHidGuid(ByRef gHid As Guid)
		End Sub

		<DllImport("hid.dll")> _
		Public Shared Function HidD_GetAttributes(HidDeviceObject As IntPtr, ByRef Attributes As HIDD_ATTRIBUTES) As [Boolean]
		End Function

		<DllImport("hid.dll")> _
		Friend Shared Function HidD_SetOutputReport(HidDeviceObject As IntPtr, lpReportBuffer As Byte(), ReportBufferLength As UInteger) As Boolean
		End Function

		<DllImport("setupapi.dll", CharSet := CharSet.Auto, SetLastError := True)> _
		Public Shared Function SetupDiGetClassDevs(ByRef ClassGuid As Guid, <MarshalAs(UnmanagedType.LPTStr)> Enumerator As String, hwndParent As IntPtr, Flags As UInt32) As IntPtr
		End Function

		'ref SP_DEVINFO_DATA devInfo,
		<DllImport("setupapi.dll", CharSet := CharSet.Auto, SetLastError := True)> _
		Public Shared Function SetupDiEnumDeviceInterfaces(hDevInfo As IntPtr, devInvo As IntPtr, ByRef interfaceClassGuid As Guid, memberIndex As Int32, ByRef deviceInterfaceData As SP_DEVICE_INTERFACE_DATA) As [Boolean]
		End Function

		<DllImport("setupapi.dll", SetLastError := True)> _
		Public Shared Function SetupDiGetDeviceInterfaceDetail(hDevInfo As IntPtr, ByRef deviceInterfaceData As SP_DEVICE_INTERFACE_DATA, deviceInterfaceDetailData As IntPtr, deviceInterfaceDetailDataSize As UInt32, ByRef requiredSize As UInt32, deviceInfoData As IntPtr) As [Boolean]
		End Function

		<DllImport("setupapi.dll", SetLastError := True)> _
		Public Shared Function SetupDiGetDeviceInterfaceDetail(hDevInfo As IntPtr, ByRef deviceInterfaceData As SP_DEVICE_INTERFACE_DATA, ByRef deviceInterfaceDetailData As SP_DEVICE_INTERFACE_DETAIL_DATA, deviceInterfaceDetailDataSize As UInt32, ByRef requiredSize As UInt32, deviceInfoData As IntPtr) As [Boolean]
		End Function

		<DllImport("setupapi.dll", CharSet := CharSet.Auto, SetLastError := True)> _
		Public Shared Function SetupDiDestroyDeviceInfoList(hDevInfo As IntPtr) As UInt16
		End Function

		<DllImport("Kernel32.dll", SetLastError := True, CharSet := CharSet.Auto)> _
		Public Shared Function CreateFile(fileName As String, <MarshalAs(UnmanagedType.U4)> fileAccess As FileAccess, <MarshalAs(UnmanagedType.U4)> fileShare As FileShare, securityAttributes As IntPtr, <MarshalAs(UnmanagedType.U4)> creationDisposition As FileMode, <MarshalAs(UnmanagedType.U4)> flags As EFileAttributes, _
			template As IntPtr) As SafeFileHandle
		End Function

		<DllImport("kernel32.dll", SetLastError := True)> _
		Public Shared Function CloseHandle(hObject As IntPtr) As <MarshalAs(UnmanagedType.Bool)> Boolean
		End Function
	End Class
End Namespace
